@@ -199,46 +199,73 @@ static void append_text_record(struct near_ndef_text_record *text,
 
 }
 
+static const char *uri_prefix[NFC_MAX_URI_ID + 1] = {
+	"",
+	"http://www.",
+	"https://www.",
+	"http://",
+	"https://",
+	"tel:",
+	"mailto:",
+	"ftp://anonymous:anonymous@",
+	"ftp://ftp.",
+	"ftps://",
+	"sftp://",
+	"smb://",
+	"nfs://",
+	"ftp://",
+	"dav://",
+	"news:",
+	"telnet://",
+	"imap:",
+	"rstp://",
+	"urn:",
+	"pop:",
+	"sip:",
+	"sips:",
+	"tftp:",
+	"btspp://",
+	"btl2cap://",
+	"btgoep://",
+	"tcpobex://",
+	"irdaobex://",
+	"file://",
+	"urn:epc:id:",
+	"urn:epc:tag:",
+	"urn:epc:pat:",
+	"urn:epc:raw:",
+	"urn:epc:",
+	"urn:nfc:",
+};
+
+const char *__near_ndef_get_uri_prefix(uint8_t id)
+{
+	if (id > NFC_MAX_URI_ID)
+		return NULL;
+
+	return uri_prefix[id];
+}
+
 static void append_uri_record(struct near_ndef_uri_record *uri,
 					DBusMessageIter *dict)
 {
-	char *value, *prefix = NULL;
+	char *value;
+	const char *prefix = NULL;
 
 	DBG("");
 
 	if (uri == NULL || dict == NULL)
 		return;
 
-	switch (uri->identifier) {
-	case 0x1:
-		prefix = "http://www.";
-		break;
-	case 0x2:
-		prefix = "https://www.";
-		break;
-	case 0x3:
-		prefix = "http://";
-		break;
-	case 0x4:
-		prefix = "https://";
-		break;
-	case 0x5:
-		prefix = "tel:";
-		break;
-	case 0x6:
-		prefix = "mailto:";
-		break;
-	case 0x7:
-		prefix = "ftp://anonymous:anonymous@";
-		break;
-	case 0x8:
-		prefix = "ftp://ftp.";
-		break;
-	case 0x9:
-		prefix = "ftps://";
-		break;
+	if (uri->identifier > NFC_MAX_URI_ID) {
+		near_error("Invalid URI identifier 0x%x", uri->identifier);
+		return;
 	}
 
+	prefix = uri_prefix[uri->identifier];
+
+	DBG("URI prefix %s", prefix);
+
 	value = g_strdup_printf("%s%.*s", prefix, uri->field_length,
 							 uri->field);
 
@@ -444,6 +471,9 @@ static void free_sp_record(struct near_ndef_sp_record *sp)
 
 static void free_mime_record(struct near_ndef_mime_record *mime)
 {
+	if (mime == NULL)
+		return;
+
 	g_free(mime->type);
 	g_free(mime);
 
@@ -679,6 +709,8 @@ static struct near_ndef_record_header *parse_record_header(uint8_t *rec,
 	struct near_ndef_record_header *rec_header = NULL;
 	uint8_t type_len, *type = NULL;
 
+	DBG("length %d", length);
+
 	if (rec == NULL || offset >= length)
 		return NULL;
 
@@ -696,6 +728,10 @@ static struct near_ndef_record_header *parse_record_header(uint8_t *rec,
 	rec_header->il = RECORD_IL_BIT(rec[offset]);
 	rec_header->tnf = RECORD_TNF_BIT(rec[offset]);
 
+	DBG("mb %d me %d sr %d il %d tnf %d",
+		rec_header->mb, rec_header->me, rec_header->sr,
+		rec_header->il, rec_header->tnf);
+
 	offset++;
 	type_len = rec[offset++];
 
@@ -710,6 +746,8 @@ static struct near_ndef_record_header *parse_record_header(uint8_t *rec,
 			goto fail;
 	}
 
+	DBG("payload length %d", rec_header->payload_len);
+
 	if (rec_header->il == 1) {
 		rec_header->il_length = rec[offset++];
 
@@ -1158,13 +1196,9 @@ parse_mime_type(struct near_ndef_record *record,
 	if (strcmp(mime->type, "application/vnd.bluetooth.ep.oob") == 0) {
 		err = __near_bt_parse_oob_record(BT_MIME_V2_1,
 				&ndef_data[offset]);
-	} else {
-		if (strcmp(mime->type, "nokia.com:bt") == 0) {
+	} else if (strcmp(mime->type, "nokia.com:bt") == 0) {
 		err = __near_bt_parse_oob_record(BT_MIME_V2_0,
 				&ndef_data[offset]);
-		}
-		else
-			err = -EOPNOTSUPP;
 	}
 
 	if (err < 0) {
